local addonInfo, privateVars = ...

---------- init namespace ---------

local uiElements  = privateVars.uiElements
local data        = privateVars.data
local _internal   = privateVars.internal

---------- init local variables ---------

---------- addon internal function block ---------

function _internal.Button()
  
  local button = nil
  local buttonSize = 48 * (nkWSetup.scale / 100)
  local iconSize = 36 * (nkWSetup.scale / 100)
  local mouseDown = false
  
  local subButtons
  
  local addonList = Inspect.Addon.List()
  
  local showButtonFlag = false
    
  button = UI.CreateFrame ('Frame', 'nkWardrobeButton', uiElements.contextSecure)
      
  button.border = UI.CreateFrame ('Texture', 'nkWardrobeButton_Border', button)
  button.border:SetTextureAsync("Rift", "HeaderButton_Normal_Down.png.dds") 
  button.border:SetPoint("TOPLEFT", button, "TOPLEFT")
  button.border:SetPoint("BOTTOMRIGHT", button, "BOTTOMRIGHT")
  button.border:SetLayer(1)
    
  if nkWSetup.border == false then
    button.border:SetVisible(false)
    button:SetBackgroundColor(0, 0, 0, 1)
    iconSize = buttonSize
  end

  button:SetWidth(buttonSize)
  button:SetHeight(buttonSize)
  button:SetPoint ("TOPRIGHT", UIParent, "TOPRIGHT", nkWSetup.buttonX, nkWSetup.buttonY)
  
  button:SetSecureMode('restricted')
  
  button:EventAttach(Event.UI.Input.Mouse.Left.Click, function (self)
    if Inspect.System.Secure() == false then
      if uiElements.tooltip ~= nil then uiElements.tooltip:SetVisible(false) end
      self:ToggleButtons()
    end
  end, "nkWardrobeButton_LeftClick")
  
  button:EventAttach(Event.UI.Input.Mouse.Right.Down, function (self)
    mouseDown = true
  end, "nkWardrobeButton_RightDown")
  
  button:EventAttach(Event.UI.Input.Mouse.Cursor.Move, function (self)
    if Inspect.System.Secure() == false and mouseDown == true then
      local mouseData = Inspect.Mouse()
      
      local curdivX = mouseData.x - button:GetLeft()
      local curdivY = mouseData.y - button:GetTop()
      
      nkWSetup.buttonX = nkWSetup.buttonX + curdivX
      nkWSetup.buttonY = nkWSetup.buttonY + curdivY
      
      button:SetPoint ("TOPRIGHT", UIParent, "TOPRIGHT", nkWSetup.buttonX, nkWSetup.buttonY)
    end
  end, "nkWardrobeButton_CursorMove")       
  
  button:EventAttach(Event.UI.Input.Mouse.Right.Up, function (self)
    mouseDown = false       
  end, "nkWardrobeButton_RightUp")      
  
  button:EventAttach(Event.UI.Input.Mouse.Right.Upoutside , function ()
    mouseDown = false
  end, "nkWardrobeButton_RightUpOutside")     
  
  button:EventAttach(Event.UI.Input.Mouse.Cursor.In, function (self)
    if Inspect.System.Secure() == false then
      self.ttCounter = 0
      self:ShowHelp()
    end
  end, "nkWardrobeButton_CursorIn")
  
  button:EventAttach(Event.UI.Input.Mouse.Cursor.Out, function (self)
    if Inspect.System.Secure() == false and 0 == self.ttCounter then
      self.ttCounter = nil
      if uiElements.tooltip ~= nil then uiElements.tooltip:SetVisible(false) end
    end
  end, "nkWardrobeButton_CursorOut")      

  if not nkWSetup.sets[nkWSetup.activeSet] then nkWSetup.activeSet = 1 end

  local icon = nkWSetup.sets[nkWSetup.activeSet].icon
  if not icon then icon = "CharacterSheet_I1DB.dds" end 

  button.icon = UI.CreateFrame ('Texture', 'nkWardrobeButtonIcon', button)
  button.icon:SetTextureAsync("Rift", icon)
  button.icon:SetWidth(iconSize)
  button.icon:SetHeight(iconSize)
  button.icon:SetPoint("CENTER", button, "CENTER")
  button.icon:SetSecureMode('restricted')
  button.icon:SetLayer(2)
    
  function button:BuildSubButtons()
  
    local buildFunc = function ()
  
      local from, object, to, x, y = nil, button, nil, 0, 0
    
      if nkWSetup.open == 'bottom' then
        from, to = "TOPLEFT", "BOTTOMLEFT"
        if nkWSetup.border == false then y = -5 end
      elseif nkWSetup.open == 'top' then  
        from, to = "BOTTOMLEFT", "TOPLEFT"
        if nkWSetup.border == false then y = 5 end
      elseif nkWSetup.open == 'left' then
        from, to = "CENTERRIGHT", "CENTERLEFT"
        if nkWSetup.border == false then x = -5 end
      elseif nkWSetup.open == 'right' then
        from, to = "CENTERLEFT", "CENTERRIGHT"
        if nkWSetup.border == false then x = 5 end
      end       
    
      local counter = 1
      if subButtons == nil then subButtons = {} end
      
      for k, v in pairs (nkWSetup.sets) do
    
        local subIconPath = v.icon
        if not subIconPath or subIconPath == "" then subIconPath = 'CharacterSheet_I1DB.dds' end
        
        local subButton = nil
      
        if subButtons[counter] == nil then
      
          subButton = UI.CreateFrame ('Frame', 'nkWardrobeButton', uiElements.contextSecure)
          
          subButton.border = UI.CreateFrame("Texture", 'nkWardrobeSubButton' .. k, subButton)
          subButton.border:SetTextureAsync('Rift', 'HeaderButton_Normal_Down.png.dds')
          subButton.border:SetLayer(1)
          subButton.border:SetPoint("TOPLEFT", subButton, "TOPLEFT")
          subButton.border:SetPoint("BOTTOMRIGHT", subButton, "BOTTOMRIGHT")
          
          if nkWSetup.border == false then subButton.border:SetVisible(false) end
          
          subButton:SetWidth(buttonSize)
          subButton:SetHeight(buttonSize)
          subButton:SetPoint(from, object, to, x, y)      
          subButton:SetVisible(false)
          subButton:SetSecureMode ('restricted')
                
          subButton.icon = UI.CreateFrame ('Texture', 'nkWardrobeButton' .. k .. 'Icon', subButton)
          subButton.icon:SetTextureAsync("Rift", subIconPath)
          subButton.icon:SetWidth(iconSize)
          subButton.icon:SetHeight(iconSize)
          subButton.icon:SetPoint("CENTER", subButton, "CENTER")
          subButton.icon:SetSecureMode('restricted')
          subButton.icon:SetLayer(2)
          
          subButton:EventAttach(Event.UI.Input.Mouse.Cursor.In, function (self)
            self.ttCounter = k
            button:ShowTooltip(k)
          end, "nkWardrobeSubButton_".. k .. "CursorIn")
          
          subButton:EventAttach(Event.UI.Input.Mouse.Cursor.Out, function (self)
            if k == self.ttCounter then
              self.ttCounter = nil
              if uiElements.tooltip~= nil then uiElements.tooltip:SetVisible(false) end
            end
          end, "nkWardrobeSubButton_".. k .. "CursorOut")
          
          table.insert (subButtons, subButton)
        else
          subButton = subButtons[counter]
          subButton:SetPoint(from, object, to, x, y)
          subButton:SetVisible (false)
          subButton.icon:SetTextureAsync("Rift", subIconPath)
        end
        
        subButton:EventDetach(Event.UI.Input.Mouse.Left.Click, nil, "nkWardrobeSubButton_".. k .. "LeftClick")
        
        subButton:EventAttach(Event.UI.Input.Mouse.Left.Click, function (self)
          if Inspect.System.Secure() == false then
            _internal.wearEquip(k)       
            button.icon:SetTexture ("Rift", subIconPath)
            button:ToggleButtons()
            if uiElements.tooltip ~= nil then uiElements.tooltip:SetVisible(false) end
          end
        end, "nkWardrobeSubButton_".. k .. "LeftClick")
        
        local macro = ""
        if v.role ~= 0 or (v.nkrebuff ~= 0 and v.nkrebuff ~= -1 ) or v.kaMain ~= -1 or v.kaSub ~= -1 then -- probably only the check for -1 of nkrebuff needed
          if not (v.role == nil or v.role == 0 ) then
            macro = string.format ('role %d', v.role)
          end
          if v.nkrebuff ~= nil and v.nkrebuff ~= -1 then
            if addonList.nkRebuff ~= nil then
              macro = macro .. string.format('\nnkRB set=%d', v.nkrebuff)         
            end   
          end
          
          if v.kaMain ~= nil and v.kaMain ~= -1 then
            if addonList.kAlert ~= nil then
              macro = macro .. string.format('\nkaruulalert set=%d', v.kaMain)          
            end
          end
          
          if v.kaSub ~= nil and v.kaSub ~= -1 then
            if addonList.kAlert ~= nil then
              macro = macro .. string.format('\nkaruulalert subset=%d', v.kaSub)          
            end
          end
        end
        
        if v.yell ~= nil and v.yell ~= "" then
          macro = macro .. string.format("\nyell %s", v.yell)
        end
        
        if macro ~= "" then subButton:EventMacroSet(Event.UI.Input.Mouse.Left.Click, macro) end
            
        object = subButton
        counter = counter + 1     
      end
      
      for idx = counter, 1, #subButtons do
        subButtons[idx]:SetVisible(false)
      end
    end
    
    EnKai.events.addInsecure(buildFunc)
    
  end
  
  function button:ToggleButtons()

    local flag = showButtonFlag   
    if flag == true then flag = false else flag = true end
    showButtonFlag = flag

    for idx = 1, #nkWSetup.sets, 1 do
      if subButtons[idx] ~= nil then subButtons[idx]:SetVisible(flag) end
    end

  end

  function button:ShowTooltip(counter)

    local set = nkWSetup.sets[counter]
    if set == nil then return end

    local from, to = "BOTTOM", "TOP"
    
    if nkWSetup.buttonY < 100 then from = "TOP" end
    
    if nkWSetup.buttonX > -250 then 
      from = from .. 'RIGHT'
      to = to .. 'LEFT'
    else
      from = from .. 'LEFT'
      to = to .. 'RIGHT'
    end
      
    if uiElements.tooltip == nil then uiElements.tooltip = EnKai.uiCreateFrame('nkTooltip', 'nkWardrobeTooltip', uiElements.tooltipContext ) end
    
    local roleText = privateVars.langTexts.fldRole .. ': ' .. set.role
    
    local roleList = Inspect.Role.List()
    for k, v in pairs (roleList) do
      local id = string.sub ( k, string.len(k) - 1)
      if tonumber(id, 16)+1 == set.role then
        roleText = privateVars.langTexts.fldRole .. ': ' .. v
        break
      end   
    end
    
    local lines = { { text = roleText } }
    
    if set.nkrebuff ~= nil and set.nkrebuff ~= -1 then
      if addonList.nkRebuff ~= nil and nkRBSets[set.nkrebuff] ~= nil and nkRBSets[set.nkrebuff].name ~= nil then
        table.insert(lines, { text = string.format("%s: %s", privateVars.langTexts.fldRebuff, nkRBSets[set.nkrebuff].name) })
      end
    end
    
    if addonList.kAlert ~= nil  and set.kaMain ~= nil and set.kaMain ~= -1 then
      if set.kaMain ~= nil and set.kaMain ~= -1 then
        table.insert(lines, { text = string.format("%s: %d", privateVars.langTexts.fldKAMain, set.kaMain) })
      end
      
      if set.kaSub ~= nil and set.kaSub ~= -1 then
        table.insert(lines, { text = string.format("%s: %d", privateVars.langTexts.fldKASub, set.kaSub) })
      end
    end
    
    table.insert(lines, { text = '', height = 8, })
    table.insert(lines, { text = privateVars.langTexts.equipTooltip })
    
    uiElements.tooltip:SetTitle(set.name)   
    uiElements.tooltip:ClearAll()
    uiElements.tooltip:SetPoint(from, subButtons[counter],  to)
    uiElements.tooltip:SetLines(lines)
    uiElements.tooltip:SetVisible(true)
    
  end

  function button:ShowHelp()
  
    local from, to = "BOTTOM", "TOP"
    
    if nkWSetup.buttonY < 100 then from = "TOP" end
    
    if nkWSetup.buttonX > -250 then 
      from = from .. 'RIGHT'
      to = to .. 'LEFT'
    else
      from = from .. 'LEFT'
      to = to .. 'RIGHT'
    end
              
    if uiElements.tooltip == nil then uiElements.tooltip = EnKai.uiCreateFrame('nkTooltip', 'nkWardrobeTooltip', uiElements.tooltipContext ) end
      
    local lines = { { text = privateVars.langTexts.helpLine1 }, { text = privateVars.langTexts.helpLine3 } }
    
    uiElements.tooltip:SetTitle(privateVars.langTexts.helpTitle)
    uiElements.tooltip:ClearAll()
    uiElements.tooltip:SetPoint(from, button,  to)
    uiElements.tooltip:SetLines(lines)
    uiElements.tooltip:SetVisible(true)

  end

  function button:Update()
  
    buttonSize = 48 * (nkWSetup.scale / 100)
  
    if nkWSetup.border == true then
      iconSize = 36 * (nkWSetup.scale / 100)
    else
      iconSize = buttonSize
    end

    button:SetWidth(buttonSize)
    button:SetHeight(buttonSize)
    button.border:SetVisible(nkWSetup.border)
    button.icon:SetWidth(iconSize)
    button.icon:SetHeight(iconSize)
    button.icon:SetTextureAsync("Rift", nkWSetup.sets[nkWSetup.activeSet].icon)
    
    for idx = 1, #subButtons, 1 do
      subButtons[idx].border:SetVisible(nkWSetup.border)
      subButtons[idx].icon:SetWidth(iconSize)
      subButtons[idx].icon:SetHeight(iconSize)
    end
  
  end
  
  function button:ReAlign()
  
    local from, object, to, x, y = nil, button, nil, 0, 0
  
    if nkWSetup.open == 'bottom' then
      from, to = "TOPLEFT", "BOTTOMLEFT"
      if nkWSetup.border == false then y = -5 end
    elseif nkWSetup.open == 'top' then  
      from, to = "BOTTOMLEFT", "TOPLEFT"
      if nkWSetup.border == false then y = 5 end
    elseif nkWSetup.open == 'left' then
      from, to = "CENTERRIGHT", "CENTERLEFT"
      if nkWSetup.border == false then x = -5 end
    elseif nkWSetup.open == 'right' then
      from, to = "CENTERLEFT", "CENTERRIGHT"
      if nkWSetup.border == false then x = 5 end
    end 
    
    for idx = 1, #subButtons, 1 do
      subButtons[idx]:ClearAll()
      subButtons[idx]:SetPoint(from, object, to, x, y)
      object = subButtons[idx]
    end
    
  end
  
  button:BuildSubButtons()  
  
  return button
  
end