local addonInfo, privateVars = ...

---------- init namespace ---------

if nkWardrobe then return end
if not nkWardrobe then nkWardrobe = {} end

privateVars.uiElements = {}
privateVars.internal = {}
privateVars.data = {}

local uiElements  = privateVars.uiElements
local data        = privateVars.data
local _internal   = privateVars.internal

---------- init local variables ---------

local _init             = false
local _lastEventUpdate  = nil

---------- init variables ---------

data.slots = {'helmet', 'cape', 'shoulders', 'chest', 'gloves', 'belt', 'legs', 'feet', 'focus', 'handmain', 'handoff', 'ranged', 'earring1', 'earring2', 'neck', 'trinket', 'ring1', 'ring2', 'synergy', 'seal' }

---------- generate ui context ---------

uiElements.context = UI.CreateContext("nkWardrobe")
uiElements.context:SetStrata ('dialog')

uiElements.contextSecure = UI.CreateContext("nkWardrobe.Secure")
uiElements.contextSecure:SetSecureMode('restricted')

uiElements.contextTop = UI.CreateContext("nkWardrobe.Dialog")
uiElements.contextTop:SetStrata ('topmost')

uiElements.tooltipContext = UI.CreateContext("nkWardrobe.Tooltip")
uiElements.tooltipContext:SetStrata ('topmost')

---------- local function block ---------

local function _fctUpdateHandler ()

  if uiElements.tooltipInfo == nil then return end
  local now = Inspect.Time.Real()

  if privateVars.lastUpdateEvent ~= nil and EnKai.tools.math.round((now - _lastEventUpdate), 1) < 1 then return end
  
  _lastEventUpdate = now

  if data.lastTTDisplay ~= nil then    
    if now - data.lastTTDisplay > 2 then
      local type, id = Inspect.Tooltip()
      if type ~= 'item' then
        uiElements.tooltipInfo:SetVisible(false)
        data.lastTTDisplay = nil
      end
    end
  end
  
end

local function _fctCommandHandler(commandline)

  if string.find(commandline, 'equip=') ~= nil then
    local pos = string.find(commandline, 'equip=')
    local selset=string.sub(commandline, pos+6, string.len(commandline))
    local setno = tonumber(selset)
       
    for k,v in pairs(nkWSetup.sets) do
      if v.name == selset then
        setno = k
        break
      end
    end
       
    if setno < 1 or setno > #nkWSetup.sets then
      EnKai.tools.error.display (addonInfo.identifier, string.format(privateVars.langTexts.commandSetRefuse, setno, #nkWSetup.sets), 2)
    else
      _internal.wearEquip (setno)
    end
  end

end

local function _fctMain(_, addon)

  if addon == addonInfo.identifier then

    if nkPanel ~= nil and nkWSetup.nkPanel == true then
      uiElements.panel = _internal.nkPanelPlugin(_internal.buildMenu())
      nkPanel.api.registerPlugin('nkWardrobe', uiElements.panel)
      uiElements.panel:SetTitle(string.format(privateVars.langTexts.activeSet, nkWSetup.activeSet, nkWSetup.sets[nkWSetup.activeSet].name))     
    else  
      EnKai.manager.init('nkWardrobe', _internal.buildMenu(), nil)
    end
  
    table.insert(Command.Slash.Register("nkWardrobe"), {_fctCommandHandler, "nkWardrobe", "commandHandler"})
    table.insert(Command.Slash.Register("nkw"), {_fctCommandHandler, "nkWardrobe", "commandHandler"})
    
    if nkWSetup.button == true and uiElements.macroButton == nil then uiElements.macroButton = _internal.Button() end  
  
    Command.Event.Attach(Event.Tooltip, _internal.tooltipInfo, "nkWardrobe.Tooltip")
    Command.Event.Attach(Event.System.Update.Begin, _fctUpdateHandler, "nkWardrobe.Event.System.Update.Begin")
    
    function UI.Native.Character.Event:Loaded()
      if uiElements.charWindow == nil then
        if Inspect.System.Secure() == false then Command.System.Watchdog.Quiet() end
        uiElements.charWindow = _internal.uiExtension()
      end
      
      if UI.Native.Character:GetLoaded() == true then _internal.updateRebuffSelectValues(uiElements.charWindow) end
      
      uiElements.charWindow:SetVisible(UI.Native.Character:GetLoaded())
      _internal.updateUIOnSetChange(privateVars.uiSelectedSet) 
    end 
                                              
    EnKai.version.init(addonInfo.toc.Identifier, addonInfo.toc.Version)
    
    if (nkWSetup.tutorial == nil) then
      uiElements.tutorial = _internal.tutorial()
      nkWSetup.tutorial = true 
    end
    
  end   
  
end

local function _fctUnitInit ()

  if _init == true then return end

  if privateVars.calling == nil then
    local userDetails = Inspect.Unit.Detail ('player')
    privateVars.calling = userDetails.calling
  end
  
  Command.Event.Detach(Event.Unit.Availability.Full, nil, "nkWardrobe.Unit.Available.Full")

  _init = true

end

local function _fctSettingsHandler(_, addon) 

  if addon == addonInfo.identifier then
  
    local settings = {  
      nkPanel = true, 
      open = 'bottom',
      border = true,
      scale = 80,
      activeSet = 1,
      button = true,
      buttonX = -190,
      buttonY = 260,
      currentSet = 1,
      sets = { {name = string.format(privateVars.langTexts.set, 1), linkedSet = -1, wardrobe = -1, kaMain = -1, kaSub = -1, yell= "", role = 1, bag = 1, bank = 0, icon = 'CharacterSheet_I1DB.dds', items = {} } }
    }
    
    if nkWSetup == nil then
      nkWSetup = settings
    else
      for k, v in pairs (settings) do if nkWSetup[k] == nil then nkWSetup[k] = v end end
    end
      
    nkWSetup.version = addonInfo.toc.Version
    
    privateVars.uiSelectedSet = nkWSetup.activeSet or 1
    
    -- Version 4.2.0 config change
    
    if nkWSetup.nkPanel == nil then nkWSetup.nkPanel = true end
    
    -- Version 5.0.0 config change
    
    for idx = 1, #nkWSetup.sets, 1 do
      if nkWSetup.sets[idx].linkedSet == nil then nkWSetup.sets[idx].linkedSet = -1 end     
      if nkWSetup.sets[idx].kaMain == nil then nkWSetup.sets[idx].kaMain = -1 end
      if nkWSetup.sets[idx].kaSub == nil then nkWSetup.sets[idx].kaSub = -1 end     
      if nkWSetup.sets[idx].yell == nil then nkWSetup.sets[idx].yell = "" end
    end
    
    -- icon change

    local icons = {
      ['ressource/liberator.png'] = "soul_liberator.dds",
      ['ressource/warrior1.png'] = "soul_beastmaster.dds",
      ['ressource/warrior2.png'] = "soul_champion.dds",
      ['ressource/warrior3.png'] = "soul_warlord.dds",
      ['ressource/warrior4.png'] = "soul_paladin.dds",
      ['ressource/warrior5.png'] = "soul_paragon.dds",
      ['ressource/warrior6.png'] = "soul_reaver.dds",
      ['ressource/warrior7.png'] = "soul_riftblade.dds",
      ['ressource/warrior8.png'] = "soul_voidknight.dds",
      ['ressource/warrior9.png'] = "soul_tempest.dds",
      ['ressource/oracle.png'] = "soul_oracle.dds",
      ['ressource/cleric1.png'] = "soul_warden.dds",
      ['ressource/cleric2.png'] = "soul_druid.dds",
      ['ressource/cleric3.png'] = "soul_inquisitor.dds",
      ['ressource/cleric4.png'] = "soul_cabalist.dds",
      ['ressource/cleric5.png'] = "soul_purifier.dds",
      ['ressource/cleric6.png'] = "soul_justicar.dds",
      ['ressource/cleric7.png'] = "soul_shaman.dds",
      ['ressource/cleric8.png'] = "soul_sentinel.dds",
      ['ressource/cleric9.png'] = "soul_defiler.dds",
      ['ressource/arbiter.png'] = "soul_arbiter.dds",
      ['ressource/mage1.png'] = "soul_archon.dds",
      ['ressource/mage2.png'] = "soul_dominator.dds",
      ['ressource/mage3.png'] = "soul_chloromancer.dds",
      ['ressource/mage4.png'] = "soul_elementalist.dds",
      ['ressource/mage5.png'] = "soul_warlock.dds",
      ['ressource/mage6.png'] = "soul_necromancer.dds",
      ['ressource/mage7.png'] = "soul_pyromancer.dds",
      ['ressource/mage8.png'] = "soul_stormcaller.dds",
      ['ressource/mage9.png'] = "soul_harbringer.dds",
      ['ressource/physician.png'] = "soul_physician.dds",
      ['ressource/rogue1.png'] = "soul_assassin.dds",
      ['ressource/rogue2.png'] = "soul_bard.dds",
      ['ressource/rogue3.png'] = "soul_bladedancer.dds",
      ['ressource/rogue4.png'] = "soul_nightblade.dds",
      ['ressource/rogue5.png'] = "soul_riftstalker.dds",
      ['ressource/rogue6.png'] = "soul_saboteur.dds",
      ['ressource/rogue7.png'] = "soul_marksman.dds",
      ['ressource/rogue8.png'] = "soul_ranger.dds",
      ['ressource/rogue9.png'] = "soul_tactician.dds"      
    }
    
    for key, data in pairs (nkWSetup.sets) do
      if data.icon == nil then
        nkWSetup.sets[key].icon = 'CharacterSheet_I1DB.dds'
      elseif string.find(data.icon, "ressource") ~= nil then 
        nkWSetup.sets[key].icon = icons[data.icon]
      end
    end
    
  end
      
end

local function _fctCalculateMoveQueue (moveSet)

  local moveList = {}
  
  if nkWSetup.sets[moveSet].items == nil then return end
  
  local oInspectItemFind = Inspect.Item.Find
  local oUtilityItemSlotEquipment = Utility.Item.Slot.Equipment
  local oInspectItemDetail = Inspect.Item.Detail
  local oStringFormat = string.format
    
  for k, v in pairs (nkWSetup.sets[moveSet].items) do
    local err, sourceSlot = pcall(oInspectItemFind, v)
    
    if err == true and sourceSlot ~= nil then
      local slot = oUtilityItemSlotEquipment(k)
      if slot ~= sourceSlot then
        moveList[slot] = sourceSlot
      else
        moveList[slot] = "NOMOVE"
      end
    else
      local itemDetails = oInspectItemDetail(v)
      if itemDetails == nil then
        EnKai.tools.error.display (addonInfo.identifier, oStringFormat(privateVars.langTexts.errGettingItem, v), 3)
      else
        EnKai.tools.error.display (addonInfo.identifier, oStringFormat(privateVars.langTexts.errGettingItem, itemDetails.name), 3)
      end
    end
  end
  
  return moveList, nil, nil 

end

local function _fctEquipMoverCoRoutine (thisList, list)

  local oCommandItemMove = Command.Item.Move

  for idx = 1, #thisList, 1 do
    local err = pcall (oCommandItemMove, thisList[idx].from, thisList[idx].to)
    if err == false then
      EnKai.tools.error.display (addonInfo.identifier, "Problem moving " .. thisList[idx].from .. ' / ' .. thisList[idx].to, 1)
    end
    coroutine.yield(idx)
  end
end

---------- addon internal function block ---------

function _internal.buildMenu()

  local items = {}
  local addonList = Inspect.Addon.List()
  
  local oStringFormat = string.format
  
  for k, v in pairs (nkWSetup.sets) do
    local macro = ""
    
    if v.role ~= 0 or v.nkrebuff ~= 0 then
      if not (v.role == nil or v.role == 0 ) then
        macro = oStringFormat('role %d', v.role)
      end
      
      if v.nkrebuff ~= nil and v.nkrebuff ~= -1 then  
        if addonList.nkRebuff ~= nil then 
          macro = macro .. oStringFormat('\nnkRB set=%d', v.nkrebuff) 
        end   
      end
    end
    
    if v.kaMain ~= nil and v.kaMain ~= -1 then
      if addonList.kAlert ~= nil then
        macro = macro .. oStringFormat('\nkaruulalert set=%d', v.kaMain)          
      end
    end
    
    if v.kaSub ~= nil and v.kaSub ~= -1 then
      if addonList.kAlert ~= nil then
        macro = macro .. oStringFormat('\nkaruulalert subset=%d', v.kaSub)          
      end
    end
    
    if v.yell ~= nil and v.yell ~= "" then
      macro = macro .. oStringFormat("\nyell %s", v.yell)
    end
    
    if macro == "" then macro = nil end -- not perfect but will do the job
    table.insert ( items, { label = v.name, macro = macro, callBack = function() _internal.wearEquip(k) end })
  end
  
  return items

end

function _internal.wearEquip (setNo)
  
  if nkWSetup.sets == nil or nkWSetup.sets[setNo] == nil then return end 

  nkWSetup.activeSet = setNo
  privateVars.uiSelectedSet = setNo
  uiElements.macroButton:Update()
  
  if nkWSetup.sets[setNo].name ~= nil then
    print(string.format(privateVars.langTexts.loadSet, nkWSetup.sets[setNo].name))
  else
    print(string.format(privateVars.langTexts.loadSet, tostring(setNo)))
  end
  
  local linkedList, linkedWList1, linkedWList2 
  
  if nkWSetup.sets[setNo].linkedSet ~= nil and nkWSetup.sets[setNo].linkedSet ~= -1 then 
    linkedList, linkedWList1, linkedWList2 = _fctCalculateMoveQueue (nkWSetup.sets[setNo].linkedSet)
  end
  
  local moveList, wList1, wList2 = _fctCalculateMoveQueue (setNo)
  
  local realMoveList1 = {}
  local realMoveList2 = {}
  local realMoveList3 = {}
  
  -- cleanup linked list to move only items not being part of master set
  
  if linkedList ~= nil then
  
    -- check if item is part of master set move queue from bag to equip slot
  
    for to, from in pairs(moveList) do
      linkedList[to] = nil
      
      if linkedWList1 ~= nil then
        if linkedWList2[to] ~= nil then
          linkedWList1[linkedWList2[to]] = nil
          linkedWList2[to] = nil
        end
      end
    end
    
    -- check if item is part of master set move queue from wardrobe to equip slot
  
    if wList1 ~= nil then
  
      for to, from in pairs(wList2) do
        linkedList[to] = nil
        
        if linkedWList1 ~= nil then
          if linkedWList2[to] ~= nil then
            linkedWList1[linkedWList2[to]] = nil
            linkedWList2[to] = nil
          end
        end
      end
    end
    
    -- move remaining move commands to master queue
    
    if linkedWList1 ~= nil then   
      for to, from in pairs(linkedWList1) do
        table.insert(realMoveList1, { from = from, to = to })
      end
      
      for to, from in pairs(linkedWList2) do
        table.insert(realMoveList2, { from = from, to = to })
      end
    end
    
    for to, from in pairs(linkedList) do
      if from ~= "NOMOVE" then
        table.insert(realMoveList3, { from = from, to = to })
      end
    end
  end
  
  -- fill master move queue
  
  if wList1 ~= nil then
    for to, from in pairs(wList1) do
      table.insert(realMoveList1, { from = from, to = to })
    end
    
    for to, from in pairs(wList2) do
      table.insert(realMoveList2, { from = from, to = to })
    end
  end
  
  for to, from in pairs(moveList) do
    if from ~= "NOMOVE" then
      table.insert(realMoveList3, { from = from, to = to })
    end
  end
  
  if #realMoveList1 == 0 then
    _internal.equipMover (realMoveList3, nil, nil)
  else
    _internal.equipMover (realMoveList1, realMoveList2, realMoveList3)
  end 
    
  if uiElements.charWindow ~= nil then
    uiElements.charWindow.setSelect:SetSelectedValue(setNo)
    _internal.updateUIOnSetChange(setNo)
  end
    
  if nkPanel ~= nil and nkWSetup.nkPanel == true then
    uiElements.panel:SetTitle(string.format(privateVars.langTexts.activeSet, nkWSetup.activeSet, nkWSetup.sets[nkWSetup.activeSet].name))
  end

end

function _internal.equipMover (moveList1, moveList2, moveList3)

  local moveCoRoutine = coroutine.create(_fctEquipMoverCoRoutine)
  
  local callBack3 = function ()
    if moveList3 == nil then return end
    
    local moveCoRoutine3 = coroutine.create(_fctEquipMoverCoRoutine)   
    EnKai.coroutines.add (  { func = moveCoRoutine3, counter = #moveList3, para1 = moveList3, para2 = 'list3', delay = 1, active = true } ) 
  end
  
  local callBack2 = function ()
    if moveList2 == nil then return end
    
    local moveCoRoutine2 = coroutine.create(_fctEquipMoverCoRoutine)   
    EnKai.coroutines.add (  { func = moveCoRoutine2, counter = #moveList2, para1 = moveList2, para2 = 'list2', delay = 1, active = true, callBack = callBack3 } ) 
  end

  EnKai.coroutines.add (  { func = moveCoRoutine, counter = #moveList1, para1 = moveList1, para2 = 'list1', active = true, callBack = callBack2 } ) 

end

function _internal.getAvailBagSlots ()

  local availSlots = {}
  
  local oUtilityItemSlotInventory = Utility.Item.Slot.Inventory
  local oInspectItemList = Inspect.Item.List
  
  for idx = 1, 8, 1 do  
    local bagSlot = oUtilityItemSlotInventory(idx)
    local bagInfo = oInspectItemList(bagSlot)
    
    for slot, details in pairs (bagInfo) do
      if details == false then table.insert (availSlots, slot) end   
    end
  end
  
  return availSlots

end

function _internal.tutorial ()

  local window = EnKai.uiCreateFrame("nkWizardInfo", "_internal.tutorial", uiElements.contextTop)
  
  window:SetupStep(1, {label = 'Creating a new set', desc= "Creating a new set in 5 simple steps", pointers = { 
                                                                                { from = {20, 20}, to = {60, 60}, text = "1) Click here to create a new set"},
                                                                                { from = {535, 20}, to = {275, 90}, text = "2) Click the pencil to name the new set - hit Enter to confirm"},
                                                                                { from = {20, 80}, to = {25, 110}, text = "3) Choose a role to switch to when activating the set"},
                                                                                { from = {535, 110}, to = {265, 130}, text = "4) Select an icon for the new set"},
                                                                                { from = {20, 150}, to = {25, 170}, text = "5) If nkRebuff is installed you can select a nkRebuff configuration to switch to if the set is activated\n\nAlternatively if kAlert is installed you can do the same for kAlert"}
                                                                                }, texture = {addon = "nkWardrobe", path = "gfx/wizard.png"}})
  window:SetupStep(2, {label = 'Storing equipment', desc= "Link your equipment with a set", pointers = { 
                                                                                { from = {20, 40}, to = {80, 90}, text = "1) Select the set to link your equipment with"},
                                                                                { from = {20, 120}, to = {240, 65}, text = "2) Toggle the slot selectors - checkboxes will be shown above your gear slots\n\nActivate the slots you want to link"},
                                                                                { from = {535, 20}, to = {270, 65}, text = "3) You can toggle all slots with this button"},
                                                                                { from = {535, 110}, to = {210, 65}, text = "4) Link the equipment with this button"} 
                                                                                }, texture = {addon = "nkWardrobe", path = "gfx/wizard.png"}})
  window:SetupStep(3, {label = 'Switching sets', desc= "Switching a set (and equipment) is very easy", pointers = {
                                                                                {  from = {20, 20}, text = "There are two ways of switching your gear: Using the nkWardrobe button or nkPanel"},
                                                                                { from = {20, 270}, to = {20, 360}, text = "Make sure the 'Show button' is active and nkWardrobe will create a set switcher button on the UI"},
                                                                                { from = {535, 20}, text = "The button if clicked will show all configured sets. Click on the set icon you want to switch to and nkWardrobe will load the gear, switch to the configured role and set nkRebuff / kAlert configuration"},                                                                                
                                                                                { from = {535, 270}, to = {100, 420}, text = "If nkPanel is installed you can alternatively do the same using the nkPanel plugin. Make sure this checkbox is activated if you want to use the nkPanel plugin"}
  }, texture = {addon = "nkWardrobe", path = "gfx/wizard.png"}})  
  window:SetupStep(4, {label = 'Advanced features', desc= "Wait! There's even more", pointers = {
                                                                                { from = {20, 20}, text = "You don't need to maintain your gear individually per nkWardrobe set!\n\nInstead you can maintain your gear on one main set and link the others to that!"},
                                                                                { from = {20, 270}, to = {20, 150}, text = "Simply specify your main set in this selection box for all other sets and these sets will use the equipment of the specified linked set"},
                                                                                { from = {535, 20}, to = {270, 65}, text = "Even with a linked set you can 'override' certain slots - for example a synergy crystal. To do this specify the link to your main equipment set and on the sub set only activate and store the slots to override just like in step 2"}
  }, texture = {addon = "nkWardrobe", path = "gfx/wizardStep4.png"}})
  window:SetupStep(5, {label = 'Storing equipment', desc= "You can automatically move your equipment from / to your bank and wardrobe", pointers = {
                                                                                { from = {20, 180}, to = {20, 230}, text = "Use this section to move the equipment of the selected set between your bank and your bags"},
                                                                                { from = {535, 270}, to = {270, 300}, text = "Use this section to move equipment from and to your wardrobe slots (at least the slots the wardrobe supports)"}
  }, texture = {addon = "nkWardrobe", path = "gfx/wizard.png"}})
  window:SetTitle("nkWardrobe Tutorial")
  window:SetTextureWidth(300)
  window:SetPoint("CENTER", UIParent, "CENTER")
  window:SetVisible(true)
  
  return window  

end

-------------------- STARTUP EVENTS --------------------

Command.Event.Attach(Event.Unit.Availability.Full, _fctUnitInit, "nkWardrobe.Unit.Available.Full")
Command.Event.Attach(Event.Addon.SavedVariables.Load.End, _fctSettingsHandler, "nkWardrobe.SavedVariables.Load.End")
Command.Event.Attach(Event.Addon.Load.End, _fctMain, "nkWardrobe.Addon.Load.End")