local addonInfo, privateVars = ...

---------- init namespace ---------

local uiElements  = privateVars.uiElements
local data        = privateVars.data
local _internal   = privateVars.internal

---------- init local variables ---------

---------- local function block ---------

local function _fctConfirmDialog (message, yesFunc)

  if uiElements.confirmDialog == nil then
    uiElements.confirmDialog = EnKai.uiCreateFrame("nkDialog", "_fctConfirmDialog", uiElements.contextTop)
    uiElements.confirmDialog:SetLayer(2)
    uiElements.confirmDialog:SetWidth(500)
    uiElements.confirmDialog:SetHeight(250)
    uiElements.confirmDialog:SetType('confirm')
  end
  
  uiElements.confirmDialog:SetMessage(message)
  uiElements.confirmDialog:SetVisible(true)
  
  Command.Event.Detach(EnKai.events._fctConfirmDialog.LeftButtonClicked, nil, "_fctConfirmDialog.LeftButtonClicked") -- delete possible earlier event handler and replace with new one
  
  Command.Event.Attach(EnKai.events._fctConfirmDialog.LeftButtonClicked, function ()
    yesFunc()
  end, "_fctConfirmDialog.LeftButtonClicked")
    
end

local function _fctMessageDialog (message)

  if uiElements.messageDialog == nil then
    uiElements.messageDialog = EnKai.uiCreateFrame("nkDialog", "nkWardrobe.messageDialog", uiElements.contextTop)
    uiElements.messageDialog:SetLayer(2)
    uiElements.messageDialog:SetWidth(500)
    uiElements.messageDialog:SetHeight(200)
    uiElements.messageDialog:SetType('message')
  end
  
  uiElements.messageDialog:SetMessage(message)
  uiElements.messageDialog:SetVisible(true)

end

local function _fctUpdateSetSelectValues() -- update set selection combo

  local selItems= {}
  local linkSets = {{ label = privateVars.langTexts.txtNoLink, value = -1 }}
  
  if nkWSetup.sets[1] == nil then nkWSetup.sets[1] = { name = string.format(privateVars.langTexts.txtEquipment, 1) } end
    
  for k, v in pairs (nkWSetup.sets) do
    table.insert (selItems, { label = v.name, value = k })
    
    if k ~= privateVars.uiSelectedSet then 
      table.insert (linkSets, { label = v.name, value = k })
    end   
  end
  
  uiElements.charWindow.setSelect:SetSelection(selItems)
  
  if #selItems > 0 then 
    uiElements.charWindow.linkSelect:SetSelection(linkSets)
  end
  
end

local function _fctClearEquip(loadToSet) -- clear equip from set storage

  local funcYes = function ()
    nkWSetup.sets[loadToSet].items = {}
  end

  _fctConfirmDialog (privateVars.langTexts.msgClearCurrentEquip, funcYes)

end

local function _fctStoreEquip(loadToSet) -- store equip into set storage

  local oUtilityItemSlotEquipment = Utility.Item.Slot.Equipment
  local oInspectItemList = Inspect.Item.List

  local funcYes = function ()
    local bank = nkWSetup.sets[loadToSet].bank
    local bag = nkWSetup.sets[loadToSet].bag
    local name = nkWSetup.sets[loadToSet].name
    
    nkWSetup.sets[loadToSet].items = {}
    
    local hasEquip = false
    
    for k, v in pairs (data.slots) do
    
      nkWSetup.sets[loadToSet].items[v] = nil
    
      if uiElements.charWindow.checkboxes[v]:GetChecked() == true then
    
        local equipSlot = oUtilityItemSlotEquipment(v)
        local itemID = oInspectItemList(equipSlot)
        if itemID ~= false then 
          nkWSetup.sets[loadToSet].items[v] = itemID
          hasEquip = true 
        end
      end
    end
    
    nkWSetup.sets[loadToSet].bank = bank
    nkWSetup.sets[loadToSet].bag = bag
    nkWSetup.sets[loadToSet].name = name
    
    if hasEquip == false then
      _fctMessageDialog(privateVars.langTexts.noItemsMessage)
    end
  end

  _fctConfirmDialog (privateVars.langTexts.msgLoadCurrentEquip, funcYes)
  
end

local function _fctGetEquipFromBank() -- move equip from bank to bag

  local setNo = uiElements.charWindow.setSelect:GetSelectedValue()
  
  local availSlots = {}
  local allSlots = nil
  
  if nkWSetup.sets[setNo].bag ~= nil then
    local bagSlot = Utility.Item.Slot.Inventory(nkWSetup.sets[setNo].bag)
    local bagInfo = Inspect.Item.List(bagSlot)
    
    for slot, details in pairs (bagInfo) do
      if details == false then table.insert (availSlots, slot) end    
    end
  end
  
  if #availSlots < 16 then  
    allSlots = Inspect.Item.List('si') 
    for slot, details in pairs (allSlots) do
      if EnKai.tools.table.isMember (availSlots, slot) == false and details == false then table.insert (availSlots, slot) end   
    end
  end
  
  local bankSlots = Inspect.Item.List('sb')
  
  if #availSlots == 0 then return end
  
  for k, v in pairs (nkWSetup.sets[setNo].items) do
    local err, retVal = pcall (Inspect.Item.Find, v)
    
    if err then
      if bankSlots[retVal] ~= nil then
        if #availSlots > 0 then 
          pcall (Command.Item.Move, retVal, availSlots[1])
          table.remove (availSlots, 1)
        end
      else
        local itemDetails = Inspect.Item.Detail(v)
        if itemDetails ~= nil then
          EnKai.tools.error.display (addonInfo.identifier, string.format(privateVars.langTexts.errGettingItem, itemDetails.name), 2)
        end
      end
    end
  end

end

local function _fctPutEquipToBank() -- move equip from bag to bank

  local setNo = uiElements.charWindow.setSelect:GetSelectedValue()

  local availSlots = {}
  local allSlots = nil
  
  local oCommandItemMove = Command.Item.Move
  local oInspectItemFind = Inspect.Item.Find
  
  if nkWSetup.sets[setNo].bank ~= nil then
  
    local bankSlot = nil
    if nkWSetup.sets[setNo].bank == 0 then 
      bankSlot = Utility.Item.Slot.Bank('main')
    else
      bankSlot = Utility.Item.Slot.Bank(nkWSetup.sets[setNo].bank)
    end

    local bankInfo = Inspect.Item.List(bankSlot)
    
    for slot, details in pairs (bankInfo) do
      if details == false then table.insert (availSlots, slot) end
    end
    
  end

  if #availSlots < 16 then  
    allSlots = Inspect.Item.List('sb') 
    for slot, details in pairs (allSlots) do
      if EnKai.tools.table.isMember (availSlots, slot) == false and details == false then table.insert (availSlots, slot) end   
    end
  end

  if #availSlots == 0 then return end
  
  local inventorySlots = Inspect.Item.List('si')
  local equipmentSlots = Inspect.Item.List('seqp')
    
  for k, v in pairs (nkWSetup.sets[setNo].items) do
    if k ~= 'name' then
      local err, retVal = pcall (oInspectItemFind, v)
      
      if err then
        if (inventorySlots[retVal] ~= nil or equipmentSlots[retVal] ~= nil) and #availSlots > 0 then 
          pcall (oCommandItemMove, retVal, availSlots[1])
          table.remove (availSlots, 1)
        end
      end
    end
  end

end

local function _fctSetAdd() -- create new set

  local noOfSets = #nkWSetup.sets
  nkWSetup.sets[noOfSets+1] = { name = string.format(privateVars.langTexts.txtEquipment, noOfSets+1), linkedSet = -1, wardrobe = -1, kaMain = -1, kaSub = -1, bag = 1, bank = 0, items = {}, role = 0, icon = '', nkrebuff = -1, yell = "" }
    
  privateVars.uiSelectedSet = noOfSets + 1
  
  _fctUpdateSetSelectValues()
  uiElements.charWindow.setSelect:SetSelectedValue( noOfSets + 1)
  
  _internal.updateUIOnSetChange(privateVars.uiSelectedSet)
  
end

local function _fctSetRemove() -- remove existing set

  if #nkWSetup.sets == 1 then 
    _fctMessageDialog(privateVars.langTexts.errLastSet)
  else
    local funcYes = function ()
      for idx = 1, #nkWSetup.sets, 1 do
        if nkWSetup.sets[idx].linkedSet == privateVars.uiSelectedSet then
          nkWSetup.sets[idx].linkedSet = -1 
        end
      end
     
      table.remove (nkWSetup.sets, privateVars.uiSelectedSet)     
      _fctUpdateSetSelectValues()
      uiElements.charWindow.setSelect:SetSelectedValue(1)
      privateVars.uiSelectedSet = 1     
      _internal.updateUIOnSetChange(privateVars.uiSelectedSet)
    end
    
    _fctConfirmDialog (privateVars.langTexts.msgRemoveSetConfirm, funcYes)
  end
  
end

local function _fctSetCopy() -- copy existing set

  local noOfSets = #nkWSetup.sets
  local newSet = EnKai.tools.table.copy (nkWSetup.sets[privateVars.uiSelectedSet])
  newSet.name = string.format(privateVars.langTexts.txtCopiedSet, nkWSetup.sets[privateVars.uiSelectedSet].name)
  
  table.insert ( nkWSetup.sets, newSet)
  
  _fctUpdateSetSelectValues()
  privateVars.uiSelectedSet = #nkWSetup.sets
  uiElements.charWindow.setSelect:SetSelectedValue(#nkWSetup.sets)
  _internal.updateUIOnSetChange(privateVars.uiSelectedSet)
  
end

local function _fctBuildTooltipInfo ()

  if uiElements.tooltipInfo ~= nil then return end

  uiElements.tooltipInfo = EnKai.uiCreateFrame('nkTooltip', '_internal.tooltipInfo', uiElements.tooltipContext)
  
  uiElements.tooltipInfo:SetTitle(privateVars.langTexts.partOfSets)
  uiElements.tooltipInfo:SetPoint("TOPCENTER", UI.Native.Tooltip, "BOTTOMCENTER", 0, 0)
  
  uiElements.tooltipInfo:SetVisible(false)
  
end

local function _fctTooltipInfo (target, title, lines)
  
  if uiElements.buttonTooltip == nil then
    uiElements.buttonTooltip = EnKai.uiCreateFrame('nkTooltip', 'nkWardrobe.char.buttonTooltip', uiElements.tooltipContext)
    uiElements.buttonTooltip:SetVisible(false)    
  end
  
  target:EventAttach(Event.UI.Input.Mouse.Cursor.In, function (self)
    uiElements.buttonTooltip:ClearAll()
    uiElements.buttonTooltip:SetPoint("TOPLEFT", target,  "BOTTOMRIGHT", 5, 5)
    uiElements.buttonTooltip:SetTitle(title)
    uiElements.buttonTooltip:SetLines(lines)
    uiElements.buttonTooltip:SetVisible(true)
  end, target:GetName() .. ".Mouse.Cursor.In")
  
  target:EventAttach(Event.UI.Input.Mouse.Cursor.Out, function (self)
    uiElements.buttonTooltip:SetVisible(false)
  end, target:GetName() .. ".Mouse.Cursor.Out") 

end

---------- addon internal function block ---------

function _internal.uiExtension ()

  local name = "nkWardrobe.char.window"
  local pointObject
  local extUI

  local labelColor = {r = 0.886, g = 0.835, b = 0.576, a = 1}
  local elementColor = {r = 0.557, g = 0.514, b = 0.357, a = 1}
  
  -- ***** WINDOW *****
  
  extUI = EnKai.uiCreateFrame("nkWindow", name, uiElements.context)
  extUI:SetCloseable(true)
  extUI:SetTitle(addonInfo.toc.Name)
  extUI:SetPoint("TOPLEFT", UI.Native.Character, "TOPRIGHT")
  extUI:SetVisible(false)
  extUI:SetLayer(1)
  extUI:SetWidth(375)
  extUI:SetHeight(658)
  extUI:SetLayer(1)

  -- ***** buttons *****
  
  extUI.btWear = UI.CreateFrame("Texture", name .. ".btWear", extUI)
  extUI.btWear:SetPoint("TOPLEFT", extUI:GetContent(), "TOPLEFT", 5, 5)
  extUI.btWear:SetTextureAsync("nkWardrobe", "gfx/btWear.png")
  extUI.btWear:SetWidth(32)
  extUI.btWear:SetHeight(32)
  
  extUI.btWear:EventAttach(Event.UI.Input.Mouse.Left.Click, function ()
    _internal.wearEquip(privateVars.uiSelectedSet) 
  end, name .. '.btWear.Mouse.Left.Click')
  
  _fctTooltipInfo (extUI.btWear, privateVars.langTexts.btWearTitle, privateVars.langTexts.btWearLines)
  
  extUI.btNewSet = UI.CreateFrame("Texture", name .. ".btNewSet", extUI)
  extUI.btNewSet:SetPoint("CENTERLEFT", extUI.btWear, "CENTERRIGHT", 20, 0)
  extUI.btNewSet:SetTextureAsync("nkWardrobe", "gfx/btNewSet.png")
  extUI.btNewSet:SetWidth(32)
  extUI.btNewSet:SetHeight(32)
  
  extUI.btNewSet:EventAttach(Event.UI.Input.Mouse.Left.Click, function ()
    _fctSetAdd()
  end, name .. '.btNewSet.Mouse.Left.Click')  
  
  _fctTooltipInfo (extUI.btNewSet, privateVars.langTexts.btNewSetTitle, privateVars.langTexts.btNewSetLines)
  
  extUI.btCopySet = UI.CreateFrame("Texture", name .. ".btCopySet", extUI)
  extUI.btCopySet:SetPoint("CENTERLEFT", extUI.btNewSet, "CENTERRIGHT", 5, 0)
  extUI.btCopySet:SetTextureAsync("nkWardrobe", "gfx/btCopy.png")
  extUI.btCopySet:SetWidth(32)
  extUI.btCopySet:SetHeight(32)
  
  extUI.btCopySet:EventAttach(Event.UI.Input.Mouse.Left.Click, function ()
    _fctSetCopy() 
  end, name .. '.btCopySet.Mouse.Left.Click') 
  
  _fctTooltipInfo (extUI.btCopySet, privateVars.langTexts.btCopySetTitle, privateVars.langTexts.btCopySetLines)
  
  extUI.btDeleteSet = UI.CreateFrame("Texture", name .. ".btDeleteSet", extUI)
  extUI.btDeleteSet:SetPoint("CENTERLEFT", extUI.btCopySet, "CENTERRIGHT", 5, 0)
  extUI.btDeleteSet:SetTextureAsync("nkWardrobe", "gfx/btDeleteSet.png")
  extUI.btDeleteSet:SetWidth(32)
  extUI.btDeleteSet:SetHeight(32)
  
  extUI.btDeleteSet:EventAttach(Event.UI.Input.Mouse.Left.Click, function ()
    _fctSetRemove()
  end, name .. '.btDeleteSet.Mouse.Left.Click')
  
  _fctTooltipInfo (extUI.btDeleteSet, privateVars.langTexts.btDeleteSetTitle, privateVars.langTexts.btDeleteSetLines)
  
  extUI.btToggleAll = UI.CreateFrame("Texture", name .. ".btToggleAll", extUI)
  extUI.btToggleAll:SetPoint("TOPRIGHT", extUI:GetContent(), "TOPRIGHT", -5, 5)
  extUI.btToggleAll:SetTextureAsync("nkWardrobe", "gfx/btToggleAll.png")
  extUI.btToggleAll:SetWidth(32)
  extUI.btToggleAll:SetHeight(32)
  
  extUI.btToggleAll:EventAttach(Event.UI.Input.Mouse.Left.Click, function ()
    local flag
  
    for k, v in pairs (extUI.checkboxes) do
      if flag == nil then 
        flag = v:GetChecked()         
        if flag == true then flag = false else flag = true end
      end
      
      v:SetChecked(flag)
    end
  end, name .. '.btToggle.Mouse.Left.Click')    
  
  _fctTooltipInfo (extUI.btToggleAll, privateVars.langTexts.btToggleAllTitle, privateVars.langTexts.btToggleAllLines) 
  
  extUI.btToggle = UI.CreateFrame("Texture", name .. ".btToggle", extUI)
  extUI.btToggle:SetPoint("CENTERRIGHT", extUI.btToggleAll, "CENTERLEFT", -5, 0)
  extUI.btToggle:SetTextureAsync("nkWardrobe", "gfx/btToggle.png")
  extUI.btToggle:SetWidth(32)
  extUI.btToggle:SetHeight(32)
  
  extUI.btToggle:EventAttach(Event.UI.Input.Mouse.Left.Click, function ()
    local flag
  
    for k, v in pairs (extUI.checkboxes) do
      if flag == nil then 
        flag = v:GetVisible()         
        if flag == true then flag = false else flag = true end
      end
      
      v:SetVisible(flag)
    end
  end, name .. '.btToggle.Mouse.Left.Click')    
  
  _fctTooltipInfo (extUI.btToggle, privateVars.langTexts.btToggleTitle, privateVars.langTexts.btToggleLines)
  
  extUI.btSave = UI.CreateFrame("Texture", name .. ".btSave", extUI)
  extUI.btSave:SetPoint("CENTERRIGHT", extUI.btToggle, "CENTERLEFT", -5, 0)
  extUI.btSave:SetTextureAsync("nkWardrobe", "gfx/btSave.png")
  extUI.btSave:SetWidth(32)
  extUI.btSave:SetHeight(32)

  extUI.btSave:EventAttach(Event.UI.Input.Mouse.Left.Click, function ()
    _fctStoreEquip(privateVars.uiSelectedSet)
  end, name .. '.btWear.Mouse.Left.Click')
  
  _fctTooltipInfo (extUI.btSave, privateVars.langTexts.btSaveTitle, privateVars.langTexts.btSaveLines)
  
  extUI.btClear = UI.CreateFrame("Texture", name .. ".btClear", extUI)
  extUI.btClear:SetPoint("CENTERRIGHT", extUI.btSave, "CENTERLEFT", -5, 0)
  extUI.btClear:SetTextureAsync("nkWardrobe", "gfx/btClear.png")
  extUI.btClear:SetWidth(32)
  extUI.btClear:SetHeight(32)

  extUI.btClear:EventAttach(Event.UI.Input.Mouse.Left.Click, function ()
    _fctClearEquip(privateVars.uiSelectedSet)
  end, name .. '.btWear.Mouse.Left.Click')
  
  _fctTooltipInfo (extUI.btClear, privateVars.langTexts.btClearTitle, privateVars.langTexts.btClearLines)

  -- ***** set selection *****
  
  -- set selection: selection box
  
  local selItems= {}
  
  if nkWSetup.sets[1] == nil then nkWSetup.sets[1] = { name = string.format(privateVars.langTexts.txtEquipment, 1) } end
    
  for k, v in pairs (nkWSetup.sets) do
    table.insert (selItems, { label = v.name, value = k })
  end
  
  extUI.setSelect = EnKai.uiCreateFrame("nkCombobox", name .. ".setSelect", extUI)
  extUI.setSelect:SetPoint("TOPLEFT", extUI.btWear, "BOTTOMLEFT", 0, 10)
  extUI.setSelect:SetLayer(11)
  extUI.setSelect:SetWidth(300)
  extUI.setSelect:SetText(privateVars.langTexts.setLabel)
  extUI.setSelect:SetLabelWidth(100)
  extUI.setSelect:SetSelection(selItems)
  extUI.setSelect:SetSelectedValue(privateVars.uiSelectedSet, false)
  extUI.setSelect:SetColor (elementColor)
  extUI.setSelect:SetLabelColor (labelColor)
  extUI.setSelect:SetColorSelected(labelColor)
  extUI.setSelect:SetColorInner({r = 0, g = 0, b= 0, a= 1})
  
  Command.Event.Attach(EnKai.events[name .. ".setSelect"].ComboChanged, function (_, newValue)
    privateVars.uiSelectedSet = newValue.value
    _internal.updateUIOnSetChange(newValue.value)
  end, name .. ".setSelect.ComboChanged")
  
  --- set selection: edit icon
  
  extUI.setEditIcon = UI.CreateFrame('Texture', name .. '.setSelectsetEditIcon', extUI)  
  extUI.setEditIcon:SetTextureAsync("nkWardrobe", "gfx/editPen.png")  
  extUI.setEditIcon:SetWidth(15)
  extUI.setEditIcon:SetHeight(15)
  extUI.setEditIcon:SetPoint("CENTERLEFT", extUI.setSelect, "CENTERRIGHT", 5, 0)
  
  extUI.setEditIcon:EventAttach(Event.UI.Input.Mouse.Left.Click, function ()
    if extUI.setNameEdit:GetVisible() == true then      
      extUI.setEditIcon:SetPoint("CENTERLEFT", extUI.setSelect, "CENTERRIGHT", 5, 0)      
      extUI.setNameEdit:SetKeyFocus(false)
      extUI.setSelect:SetComboVisible(true)
      extUI.setNameEdit:SetVisible(false)
    else
      extUI.setNameEdit:SetText(extUI.setSelect:GetSelectedLabel())
      extUI.setEditIcon:SetPoint ("CENTERLEFT", extUI.setNameEdit, "CENTERRIGHT", 5, 0)
      extUI.setSelect:SetComboVisible(false)
      extUI.setNameEdit:SetVisible(true)
    end
  end, name .. ".setSelectsetEditIcon.Left.Click")
  
  -- set selection: edit field
  
  extUI.setNameEdit = EnKai.uiCreateFrame("nkTextfield", name .. ".setNameEdit", extUI)
  extUI.setNameEdit:SetVisible(false)
  extUI.setNameEdit:SetWidth(200)
  extUI.setNameEdit:SetHeight(20)
  extUI.setNameEdit:SetText(nkWSetup.sets[1].name)
  extUI.setNameEdit:SetPoint("CENTERRIGHT", extUI.setSelect, "CENTERRIGHT")
  extUI.setNameEdit:SetColor(elementColor)
  extUI.setNameEdit:SetInnerColor({r = 0, g = 0, b = 0, a = 1})
  
  Command.Event.Attach(EnKai.events[name .. ".setNameEdit"].TextfieldChanged, function ()   
    nkWSetup.sets[privateVars.uiSelectedSet].name = extUI.setNameEdit:GetText()
    _fctUpdateSetSelectValues()
    
    extUI.setEditIcon:SetPoint("CENTERLEFT", extUI.setSelect, "CENTERRIGHT", 5, 0)
    extUI.setSelect:SetComboVisible(true)
    extUI.setNameEdit:SetVisible(false)
    
    if uiElements.macroButton ~= nil then uiElements.macroButton:BuildSubButtons() end    
  end, name .. ".setNameEdit.TextfieldChanged")
    
  -- ***** role selection *****
      
  local selValues = { { label = privateVars.langTexts.roleNon , value = 0 }}
  
  local roleList = Inspect.Role.List()
  for k, v in pairs (roleList) do
    local id = string.sub ( k, string.len(k) - 1)
    if id ~= nil then
      table.insert(selValues, { label = v, value = tonumber(id, 16)+1})
    end   
  end 
  
  -- role selection: selection box
  
  extUI.roleSelect = EnKai.uiCreateFrame("nkCombobox", name .. ".roleSelect", extUI)
  extUI.roleSelect:SetPoint("TOPLEFT", extUI.setSelect, "BOTTOMLEFT", 0, 5)
  extUI.roleSelect:SetWidth(300)
  extUI.roleSelect:SetLabelWidth(100)
  extUI.roleSelect:SetText(privateVars.langTexts.fldRole)
  extUI.roleSelect:SetSelection(selValues)
  extUI.roleSelect:SetSelectedValue(nkWSetup.sets[privateVars.uiSelectedSet].role, false)
  extUI.roleSelect:SetLayer(10)
  extUI.roleSelect:SetColor (elementColor)
  extUI.roleSelect:SetLabelColor (labelColor)
  extUI.roleSelect:SetColorSelected (labelColor)
  extUI.roleSelect:SetColorInner({r = 0, g = 0, b= 0, a= 1})
  
  Command.Event.Attach(EnKai.events[name .. ".roleSelect"].ComboChanged, function (_, newValue)
    nkWSetup.sets[privateVars.uiSelectedSet].role = newValue.value
    if uiElements.macroButton ~= nil then uiElements.macroButton:BuildSubButtons() end
  end, name .. ".roleSelect.ComboChanged")
  
  -- ***** icon selection *****
    
  -- icon selection: selection box
  
  local selValues = privateVars.langTexts.souls[privateVars.calling]
  table.insert(selValues, { label = privateVars.langTexts.msgChoose, value = '', iconPath = '', iconType = '' })
  
  local slot = Utility.Item.Slot.All()
  local items = Inspect.Item.Detail (slot)

  local temp = {}

  for k, v in pairs (items) do
    if temp[v.name] == nil then
      table.insert (selValues, { label = v.name, value = v.icon, iconPath = v.icon, iconType = 'Rift' })
      temp[v.name] = true
    end
  end
  
  extUI.iconSelect = EnKai.uiCreateFrame("nkCombobox", name .. ".iconSelect", extUI)
  extUI.iconSelect:SetPoint("TOPLEFT", extUI.roleSelect, "BOTTOMLEFT", 0, 5)
  extUI.iconSelect:SetWidth(300)
  extUI.iconSelect:SetLabelWidth(100)
  extUI.iconSelect:SetText(privateVars.langTexts.fldIcon)
  extUI.iconSelect:SetSelection(selValues)
  extUI.iconSelect:SetSelectedValue(nkWSetup.sets[privateVars.uiSelectedSet].icon, false)
  extUI.iconSelect:SetLayer(9)
  extUI.iconSelect:SetColor (elementColor)
  extUI.iconSelect:SetLabelColor (labelColor)
  extUI.iconSelect:SetColorSelected(labelColor)
  extUI.iconSelect:SetColorInner({r = 0, g = 0, b= 0, a= 1})
  
  Command.Event.Attach(EnKai.events[name .. ".iconSelect"].ComboChanged, function (_, newValue)
    nkWSetup.sets[privateVars.uiSelectedSet].icon = newValue.iconPath
    if uiElements.macroButton ~= nil then uiElements.macroButton:BuildSubButtons() end
  end, name .. ".iconSelect.ComboChanged")
  
  -- link selection: selection box
  
  local selSets = {{ label = privateVars.langTexts.txtNoLink, value = -1 }}
  
  for k, v in pairs (nkWSetup.sets) do
    table.insert (selSets, { label = v.name, value = k })
  end
  
  extUI.linkSelect = EnKai.uiCreateFrame("nkCombobox", name .. '.linkSelect', extUI)
  extUI.linkSelect:SetPoint("TOPLEFT", extUI.iconSelect, "BOTTOMLEFT", 0, 5)
  extUI.linkSelect:SetWidth(300)
  extUI.linkSelect:SetLabelWidth(100)
  extUI.linkSelect:SetText(privateVars.langTexts.fldLinkSelect)
  extUI.linkSelect:SetSelection(selSets)
  extUI.linkSelect:SetSelectedValue(nkWSetup.sets[privateVars.uiSelectedSet].linkedSet, false)
  extUI.linkSelect:SetLayer(8)
  extUI.linkSelect:SetColor (elementColor)
  extUI.linkSelect:SetLabelColor (labelColor)
  extUI.linkSelect:SetColorSelected(labelColor)
  extUI.linkSelect:SetColorInner({r = 0, g = 0, b= 0, a= 1})
  
  Command.Event.Attach(EnKai.events[name .. '.linkSelect'].ComboChanged, function (_, newValue)
    nkWSetup.sets[privateVars.uiSelectedSet].linkedSet = newValue.value
    _internal.updateUIOnSetChange(privateVars.uiSelectedSet)
  end, name .. '.linkSelect.ComboChanged')

  pointObject = extUI.linkSelect

  -- ***** nkRebuff integration *****
  
  local addonList = Inspect.Addon.List()
  if addonList.nkRebuff ~= nil then 
  
    extUI.nkRBSelect = EnKai.uiCreateFrame("nkCombobox", name .. ".nkRBSelect", extUI)
    extUI.nkRBSelect:SetPoint("TOPLEFT", extUI.linkSelect, "BOTTOMLEFT", 0, 5)
    extUI.nkRBSelect:SetWidth(300)
    extUI.nkRBSelect:SetLabelWidth(100)
    extUI.nkRBSelect:SetText(privateVars.langTexts.fldRebuff)
    extUI.nkRBSelect:SetLayer(7)
    extUI.nkRBSelect:SetColor (elementColor)
    extUI.nkRBSelect:SetLabelColor (labelColor)
    extUI.nkRBSelect:SetColorSelected(labelColor)
    extUI.nkRBSelect:SetColorInner({r = 0, g = 0, b= 0, a= 1})
    
    _internal.updateRebuffSelectValues(extUI)
      
    Command.Event.Attach(EnKai.events[name .. ".nkRBSelect"].ComboChanged, function (_, newValue)
      nkWSetup.sets[privateVars.uiSelectedSet].nkrebuff = newValue.value
      if uiElements.macroButton ~= nil then uiElements.macroButton:BuildSubButtons() end
    end, name .. ".nkRBSelect.ComboChanged")

    pointObject = extUI.nkRBSelect
  end
  
  if addonList.kAlert ~= nil then
    local selSets = {{ label = privateVars.langTexts.txtDeactivated, value = -1 }}
  
    for idx = 1, 20, 1 do
      table.insert (selSets, { label = string.format(privateVars.langTexts.txtKAlertSet, idx), value = idx })
    end
      
    extUI.kAlertMainSelect = EnKai.uiCreateFrame("nkCombobox", name .. ".kAlertMainSelect", extUI)
    extUI.kAlertMainSelect:SetPoint("TOPLEFT", pointObject, "BOTTOMLEFT", 0, 5)
    extUI.kAlertMainSelect:SetWidth(300)
    extUI.kAlertMainSelect:SetLabelWidth(100)
    extUI.kAlertMainSelect:SetText(privateVars.langTexts.fldKAMain)
    extUI.kAlertMainSelect:SetLayer(6)
    extUI.kAlertMainSelect:SetColor (elementColor)
    extUI.kAlertMainSelect:SetLabelColor (labelColor)
    extUI.kAlertMainSelect:SetSelection(selSets)
    extUI.kAlertMainSelect:SetSelectedValue(nkWSetup.sets[privateVars.uiSelectedSet].kaMain, false)
    extUI.kAlertMainSelect:SetColorSelected(labelColor)
    extUI.kAlertMainSelect:SetColorInner({r = 0, g = 0, b= 0, a= 1})
    
    Command.Event.Attach(EnKai.events[name .. '.kAlertMainSelect'].ComboChanged, function (_, newValue)
      nkWSetup.sets[privateVars.uiSelectedSet].kaMain = newValue.value
      if uiElements.macroButton ~= nil then uiElements.macroButton:BuildSubButtons() end
    end, name .. ".kAlertMainSelect.ComboChanged")
    
    local selSets = {{ label = privateVars.langTexts.txtDeactivated, value = -1 }}
  
    for idx = 1, 10, 1 do
      table.insert (selSets, { label = string.format(privateVars.langTexts.txtKAlertSet, idx), value = idx })
    end
    
    extUI.kAlertSubSelect = EnKai.uiCreateFrame("nkCombobox", name .. ".kAlertSubSelect", extUI)
    extUI.kAlertSubSelect:SetPoint("TOPLEFT", extUI.kAlertMainSelect, "BOTTOMLEFT", 0, 5)
    extUI.kAlertSubSelect:SetWidth(300)
    extUI.kAlertSubSelect:SetLabelWidth(100)
    extUI.kAlertSubSelect:SetText(privateVars.langTexts.fldKASub)
    extUI.kAlertSubSelect:SetLayer(5)
    extUI.kAlertSubSelect:SetColor (elementColor)
    extUI.kAlertSubSelect:SetLabelColor (labelColor)
    extUI.kAlertSubSelect:SetSelection(selSets)
    extUI.kAlertSubSelect:SetSelectedValue(nkWSetup.sets[privateVars.uiSelectedSet].kaSub, false)
    extUI.kAlertSubSelect:SetColorSelected(labelColor)
    extUI.kAlertSubSelect:SetColorInner({r = 0, g = 0, b= 0, a= 1})
    
    Command.Event.Attach(EnKai.events[name .. '.kAlertSubSelect'].ComboChanged, function (_, newValue)
      nkWSetup.sets[privateVars.uiSelectedSet].kaSub = newValue.value
      if uiElements.macroButton ~= nil then uiElements.macroButton:BuildSubButtons() end
    end, name .. ".kAlertSubSelect.ComboChanged")

    pointObject = extUI.kAlertSubSelect
    
  end 
  
  -- ***** Yell *****
  
  extUI.yellLabel = EnKai.uiCreateFrame("nkText", name .. ".yellLabel", extUI)
  extUI.yellLabel:SetWidth(100)
  extUI.yellLabel:SetPoint("TOPLEFT", pointObject, "BOTTOMLEFT", 0, 5)
  extUI.yellLabel:SetText(privateVars.langTexts.txtYell)
  extUI.yellLabel:SetFontColor (labelColor.r , labelColor.g, labelColor.b)
    
  extUI.yellEdit = EnKai.uiCreateFrame("nkTextfield", name .. ".yellEdit", extUI)
  extUI.yellEdit:SetWidth(200)
  extUI.yellEdit:SetHeight(20)
  extUI.yellEdit:SetText(nkWSetup.sets[1].yell)
  extUI.yellEdit:SetPoint("CENTERLEFT", extUI.yellLabel, "CENTERRIGHT")
  extUI.yellEdit:SetColor(elementColor)
  extUI.yellEdit:SetInnerColor({r = 0, g = 0, b = 0, a = 1})
  
  Command.Event.Attach(EnKai.events[name .. ".yellEdit"].TextfieldChanged, function ()   
    nkWSetup.sets[privateVars.uiSelectedSet].yell = extUI.yellEdit:GetText()
    if uiElements.macroButton ~= nil then uiElements.macroButton:BuildSubButtons() end    
  end, name .. ".yellEdit.TextfieldChanged")
  
  extUI.divider2 = UI.CreateFrame("Frame", name .. ".divider2", extUI)
  extUI.divider2:SetBackgroundColor(elementColor.r, elementColor.g, elementColor.b, elementColor.a)
  extUI.divider2:SetPoint("TOPLEFT", extUI.yellLabel, "BOTTOMLEFT", 0, 10)
  extUI.divider2:SetWidth(extUI:GetWidth() -40)
  extUI.divider2:SetHeight(1)
  
  -- ***** bag & bank ***** 
  
  extUI.btGetFromBank = UI.CreateFrame("Texture", name .. ".btGetFromBank", extUI)
  extUI.btGetFromBank:SetPoint("TOPLEFT", extUI.divider2, "BOTTOMLEFT", 0, 10)
  extUI.btGetFromBank:SetTextureAsync("nkWardrobe", "gfx/btGetFromBank.png")
  extUI.btGetFromBank:SetWidth(32)
  extUI.btGetFromBank:SetHeight(32)
  
  extUI.btGetFromBank:EventAttach(Event.UI.Input.Mouse.Left.Click, function ()
     _fctGetEquipFromBank()
  end, name .. '.btGetFromBank.Mouse.Left.Click')
  
  _fctTooltipInfo (extUI.btGetFromBank, privateVars.langTexts.btGetFromBankTitle, privateVars.langTexts.btGetFromBankLines)
  
  extUI.btPutToBank = UI.CreateFrame("Texture", name .. ".btPutToBank", extUI)
  extUI.btPutToBank:SetPoint("CENTERLEFT", extUI.btGetFromBank, "CENTERRIGHT", 5, 0)
  extUI.btPutToBank:SetTextureAsync("nkWardrobe", "gfx/btPutToBank.png")
  extUI.btPutToBank:SetWidth(32)
  extUI.btPutToBank:SetHeight(32)

  extUI.btPutToBank:EventAttach(Event.UI.Input.Mouse.Left.Click, function ()
     _fctPutEquipToBank()
  end, name .. '.btPutToBank.Mouse.Left.Click')
  
  _fctTooltipInfo (extUI.btPutToBank, privateVars.langTexts.btPutToBankTitle, privateVars.langTexts.btPutToBankLines)
  
  -- bag & bank: bag slot selection
  
  local selValues = {}
  
  for idx = 1, 8, 1 do 
    table.insert(selValues, { label = string.format ( privateVars.langTexts.txtBag, idx), value = idx })
  end
  
  extUI.bagSelect = EnKai.uiCreateFrame("nkCombobox", name .. ".bagSelect", extUI)
  extUI.bagSelect:SetPoint("TOPLEFT", extUI.btGetFromBank, "BOTTOMLEFT", 0, 10)
  extUI.bagSelect:SetSelection(selValues)
  extUI.bagSelect:SetWidth(300)
  extUI.bagSelect:SetLabelWidth(100)
  extUI.bagSelect:SetText(privateVars.langTexts.fldBagSelect)
  extUI.bagSelect:SetSelectedValue(nkWSetup.sets[privateVars.uiSelectedSet].bag, false)
  extUI.bagSelect:SetLayer(4)
  extUI.bagSelect:SetColor (elementColor)
  extUI.bagSelect:SetLabelColor (labelColor)
  extUI.bagSelect:SetColorSelected(labelColor)
  extUI.bagSelect:SetColorInner({r = 0, g = 0, b= 0, a= 1})
  
  Command.Event.Attach(EnKai.events[name .. ".bagSelect"].ComboChanged, function (_, newValue)
    nkWSetup.sets[privateVars.uiSelectedSet].bag = newValue.value
  end, name .. ".bagSelect.ComboChanged")
  
  -- bag & bank: bank slot selection
  
  local selValues = {}
  table.insert(selValues, { label = privateVars.langTexts.mainBankSlot, value = 0 })
  
  for idx = 1, 8, 1 do 
    table.insert(selValues, { label = string.format ( privateVars.langTexts.txtBank, idx ), value = idx })
  end 
  
  extUI.bankSelect = EnKai.uiCreateFrame("nkCombobox", name .. ".bankSelect", extUI)
  extUI.bankSelect:SetPoint("TOPLEFT", extUI.bagSelect, "BOTTOMLEFT", 0, 5)
  extUI.bankSelect:SetSelection(selValues)
  extUI.bankSelect:SetSelectedValue(nkWSetup.sets[privateVars.uiSelectedSet].bank, false)
  extUI.bankSelect:SetWidth(300)
  extUI.bankSelect:SetLabelWidth(100)
  extUI.bankSelect:SetText(privateVars.langTexts.fldBankSelect)
  extUI.bankSelect:SetLayer(3)
  extUI.bankSelect:SetColor (elementColor)
  extUI.bankSelect:SetLabelColor (labelColor)
  extUI.bankSelect:SetColorSelected(labelColor)
  extUI.bankSelect:SetColorInner({r = 0, g = 0, b= 0, a= 1})

  Command.Event.Attach(EnKai.events[name .. ".bankSelect"].ComboChanged, function (_, newValue)
    nkWSetup.sets[privateVars.uiSelectedSet].bank = newValue.value
  end, name .. ".bankSelect.ComboChanged")
  
  extUI.divider3 = UI.CreateFrame("Frame", name .. ".divider3", extUI)
  extUI.divider3:SetBackgroundColor(elementColor.r, elementColor.g, elementColor.b, elementColor.a)
  extUI.divider3:SetPoint("TOPLEFT", extUI.bankSelect, "BOTTOMLEFT", 0, 10)
  extUI.divider3:SetWidth(extUI:GetWidth() -40)
  extUI.divider3:SetHeight(1)
  
  -- ***** basic configuration *****
  
  -- basic configuration: show button checkbox
  
  extUI.ShowButtonCheckbox = EnKai.uiCreateFrame("nkCheckbox", name .. ".ShowButtonCheckbox", extUI) 
  extUI.ShowButtonCheckbox:SetPoint("TOPLEFT", extUI.divider3, "BOTTOMLEFT", 0, 15)
  extUI.ShowButtonCheckbox:SetText(privateVars.langTexts.cbButton)
  extUI.ShowButtonCheckbox:SetChecked(nkWSetup.button)
  extUI.ShowButtonCheckbox:SetLabelColor(labelColor)
  extUI.ShowButtonCheckbox:SetColor(elementColor)
  extUI.ShowButtonCheckbox:SetColorInner({r = 0, g = 0, b = 0, a = 1})
  
  Command.Event.Attach(EnKai.events[name .. ".ShowButtonCheckbox"].CheckboxChanged, function (_, newValue)    
    nkWSetup.button = newValue
    if newValue == true and uiElements.macroButton == nil then uiElements.macroButton = _internal.Button() end
    if uiElements.macroButton ~= nil then uiElements.macroButton:SetVisible(nkWSetup.button) end
  end, name .. ".ShowButtonCheckbox.CheckboxChanged")
  
  -- basic configuration: show border checkbox
  
  extUI.ShowBorderCheckbox = EnKai.uiCreateFrame("nkCheckbox", name .. ".ShowBorderCheckbox", extUI) 
  extUI.ShowBorderCheckbox:SetPoint("CENTERLEFT", extUI.ShowButtonCheckbox, "CENTERRIGHT", 10, 0)
  extUI.ShowBorderCheckbox:SetText(privateVars.langTexts.cbBorder)
  extUI.ShowBorderCheckbox:SetChecked(nkWSetup.border)
  extUI.ShowBorderCheckbox:SetLabelColor(labelColor)
  extUI.ShowBorderCheckbox:SetColor(elementColor)
  extUI.ShowBorderCheckbox:SetColorInner({r = 0, g = 0, b = 0, a = 1})
  
  Command.Event.Attach(EnKai.events[name .. ".ShowBorderCheckbox"].CheckboxChanged, function (_, newValue)    
    nkWSetup.border = newValue
    if uiElements.macroButton ~= nil then uiElements.macroButton:Update() end
  end, name .. ".ShowBorderCheckbox.CheckboxChanged")
  
  -- basic configuration: open direction select
  
  local selValues = {{ label = privateVars.langTexts.openTop, value = 'top'}, { label = privateVars.langTexts.openBottom, value = 'bottom'}, { label = privateVars.langTexts.openLeft, value = 'left'}, { label =privateVars.langTexts.openRight, value = 'right'}}
    
  extUI.openDirSelect = EnKai.uiCreateFrame("nkCombobox", name .. ".openDirSelect", extUI)
  extUI.openDirSelect:SetPoint("TOPLEFT", extUI.ShowButtonCheckbox, "BOTTOMLEFT", 0, 8)
  extUI.openDirSelect:SetSelection(selValues)
  extUI.openDirSelect:SetSelectedValue(nkWSetup.open, false)
  extUI.openDirSelect:SetWidth(300)
  extUI.openDirSelect:SetLabelWidth(100)
  extUI.openDirSelect:SetText(privateVars.langTexts.fldOpen)
  extUI.openDirSelect:SetLayer(1)
  extUI.openDirSelect:SetColor (elementColor)
  extUI.openDirSelect:SetLabelColor (labelColor)
  extUI.openDirSelect:SetColorSelected(labelColor)
  extUI.openDirSelect:SetColorInner({r = 0, g = 0, b= 0, a= 1})

  Command.Event.Attach(EnKai.events[name .. ".openDirSelect"].ComboChanged, function (_, newValue)
    nkWSetup.open = newValue.value
    if uiElements.macroButton ~= nil then uiElements.macroButton:ReAlign() end
  end, name .. ".openDirSelect.ComboChanged")

  -- basic configuration: scale slider
  
  extUI.scaleSlider = EnKai.uiCreateFrame("nkSlider", name .. ".scaleSlider", extUI)
  extUI.scaleSlider:SetPoint("TOPLEFT", extUI.openDirSelect, "BOTTOMLEFT", 0, 5)
  extUI.scaleSlider:SetWidth(200)
  extUI.scaleSlider:SetRange(0, 150)
  extUI.scaleSlider:SetLabelWidth(100)
  extUI.scaleSlider:SetText(privateVars.langTexts.slScale)
  extUI.scaleSlider:AdjustValue(nkWSetup.scale)
  extUI.scaleSlider:SetLabelColor (labelColor)
  extUI.scaleSlider:SetColor(elementColor)
  extUI.scaleSlider:SetColorInner({r = 0, g = 0, b = 0, a = 1})
  
  Command.Event.Attach(EnKai.events[name .. ".scaleSlider"].SliderChanged, function (_, newValue)
    nkWSetup.scale = newValue
    if uiElements.macroButton ~= nil then uiElements.macroButton:Update() end   
  end, name .. ".scaleSlider.SliderChanged")
  
  -- basic configuration: nkPanel
  
  extUI.nkPanelCheckbox = EnKai.uiCreateFrame("nkCheckbox", name .. ".nkPanelCheckbox", extUI) 
  extUI.nkPanelCheckbox:SetPoint("TOPLEFT", extUI.scaleSlider, "BOTTOMLEFT", 0, 15)
  extUI.nkPanelCheckbox:SetText(privateVars.langTexts.cbnkPanel)
  extUI.nkPanelCheckbox:SetChecked(nkWSetup.nkPanel)
  extUI.nkPanelCheckbox:SetLabelColor(labelColor)
  extUI.nkPanelCheckbox:SetColor(elementColor)
  extUI.nkPanelCheckbox:SetColorInner({r = 0, g = 0, b = 0, a = 1})
  
  Command.Event.Attach(EnKai.events[name .. ".nkPanelCheckbox"].CheckboxChanged, function (_, newValue)   
    nkWSetup.nkPanel = newValue
    
    if nkPanel ~= nil then
      if uiElements.panel ~= nil then
        if nkPanel.api.togglePanel ~= nil then -- nkPanel V1.1.1 check, can be removed later
          nkPanel.api.togglePanel('nkWardrobe', newValue)
        end
      elseif newValue == true then
        uiElements.panel = _internal.nkPanelPlugin(_internal.buildMenu())
        nkPanel.api.registerPlugin('nkWardrobe', uiElements.panel)
        uiElements.panel:SetTitle(string.format(privateVars.langTexts.activeSet, nkWSetup.activeSet, nkWSetup.sets[nkWSetup.activeSet].name))   
      end
    end
    
  end, name .. ".nkPanelCheckbox.CheckboxChanged")
  
  -- bottom button array: version text
  
  extUI.tutorialButton = UI.CreateFrame("RiftButton", name .. ".tutorialButton", extUI)
  extUI.tutorialButton:SetPoint("BOTTOMRIGHT", extUI, "BOTTOMRIGHT", -15, -60)
  extUI.tutorialButton:SetText("Tutorial")
  
  extUI.tutorialButton:EventAttach(Event.UI.Input.Mouse.Left.Click, function ()
    if uiElements.tutorial == nil then 
      uiElements.tutorial = _internal.tutorial() 
    end
    
    uiElements.tutorial:SetVisible(true)
  end, "nkWardrobe.cw_tutorialButton_LeftClick")
  
  extUI.EnKaiLogo = EnKai.uiCreateFrame("nkTexture", name .. ".logo", extUI)
  extUI.EnKaiLogo:SetTextureAsync(EnKai.art.GetThemeLogo()[1],EnKai.art.GetThemeLogo()[2])
  extUI.EnKaiLogo:SetPoint("BOTTOMLEFT", extUI:GetContent(), "BOTTOMLEFT", 10, -5)
  extUI.EnKaiLogo:SetWidth(125)
  extUI.EnKaiLogo:SetHeight(33)
  
  extUI.versionText = UI.CreateFrame("Text", name .. ".versionText", extUI)
  extUI.versionText:SetFontSize(11)
  extUI.versionText:SetText(string.format(privateVars.langTexts.txtHeader, addonInfo.toc.Version))
  extUI.versionText:SetFontColor(236, 228, 189, 1)
  extUI.versionText:SetPoint("BOTTOMRIGHT", extUI:GetContent(), "BOTTOMRIGHT", -10, -5)

  -- ***** Char Window *****
  
  local boxes = { {'helmet', 'cape', 'shoulders', 'chest', 'gloves', 'belt', 'legs', 'feet'}, 
                  {'handmain', 'handoff', 'ranged'},
                  {'earring1', 'earring2', 'neck', 'trinket', 'ring1', 'ring2', 'synergy', 'seal'},
                  {'focus'}
                }
  
  local from, to, object, x, y = "TOPLEFT", "TOPLEFT", UI.Native.Character, 129, 115
  local checkboxes = {}   
  
  for idx = 1, #boxes[1], 1 do
    local thisBox = EnKai.uiCreateFrame("nkCheckbox", name .. ".equipBox.1." .. boxes[1][idx], uiElements.contextTop)
    if nkWSetup.sets[privateVars.uiSelectedSet].items[boxes[1][idx]] == nil then
      thisBox:SetChecked(false)
    else
      thisBox:SetChecked(true)
    end
    thisBox:SetPoint(from, object, to, x, y)
    thisBox:SetVisible(false)
    thisBox:SetLayer(1)
    thisBox:SetColor(elementColor)
    thisBox:SetColorInner({r = 0, g = 0, b = 0, a = 1})
    from, to, object, x, y = "TOPLEFT", "BOTTOMLEFT", thisBox, 0, 40
    checkboxes[boxes[1][idx]] = thisBox 
  end
  
  from, to, object, x, y = "TOPLEFT", "TOPLEFT", UI.Native.Character, 270 , 485
  
  for idx = 1, #boxes[2], 1 do
    local thisBox = EnKai.uiCreateFrame("nkCheckbox", name .. ".equipBox.2." .. boxes[2][idx], uiElements.contextTop)
    if nkWSetup.sets[privateVars.uiSelectedSet].items[boxes[2][idx]] == nil then
      thisBox:SetChecked(false)
    else
      thisBox:SetChecked(true)
    end
    thisBox:SetVisible(false)
    thisBox:SetLayer(1)
    thisBox:SetColor(elementColor)
    thisBox:SetColorInner({r = 0, g = 0, b = 0, a = 1})
    thisBox:SetPoint(from, object, to, x, y)    
    from, to, object, x, y = "TOPLEFT", "TOPLEFT", thisBox, 55, 0
    checkboxes[boxes[2][idx]] = thisBox
  end
  
  from, to, object, x, y = "TOPLEFT", "TOPLEFT", UI.Native.Character, 523 , 115
  
  for idx = 1, #boxes[3], 1 do
    local thisBox = EnKai.uiCreateFrame("nkCheckbox", name .. ".equipBox.2." .. boxes[3][idx], uiElements.contextTop)
    if nkWSetup.sets[privateVars.uiSelectedSet].items[boxes[3][idx]] == nil then
      thisBox:SetChecked(false)
    else
      thisBox:SetChecked(true)
    end
    thisBox:SetPoint(from, object, to, x, y)
    thisBox:SetVisible(false)
    thisBox:SetLayer(1)
    thisBox:SetColor(elementColor)
    thisBox:SetColorInner({r = 0, g = 0, b = 0, a = 1})
    from, to, object, x, y = "TOPLEFT", "BOTTOMLEFT", thisBox, 0, 40
    checkboxes[boxes[3][idx]] = thisBox
  end
  
  local thisBox = EnKai.uiCreateFrame("nkCheckbox", name .. ".equipBox.3." .. boxes[4][1], uiElements.contextTop)
  if nkWSetup.sets[privateVars.uiSelectedSet].items[boxes[4][1]] == nil then
    thisBox:SetChecked(false)
  else
    thisBox:SetChecked(true)
  end
  thisBox:SetPoint("TOPLEFT", UI.Native.Character, "TOPLEFT", 323, 562)
  thisBox:SetVisible(false)
  thisBox:SetLayer(1)
  thisBox:SetColor(elementColor)
  thisBox:SetColorInner({r = 0, g = 0, b = 0, a = 1})
  checkboxes[boxes[4][1]] = thisBox
      
  extUI.checkboxes = checkboxes
  
  local oSetVisible = extUI.SetVisible
  
  function extUI:SetVisible(flag)
    oSetVisible(self, flag)
    
    if uiElements.buttonTooltip ~= nil then uiElements.buttonTooltip:SetVisible(false) end 
    
    if flag == false then
      for k, v in pairs (extUI.checkboxes) do
        v:SetVisible(false)
      end
    end
  end
  
  return extUI
  
end

function _internal.updateUIOnSetChange(selSet) -- update ui elements on set select change --

  _fctUpdateSetSelectValues()

  uiElements.charWindow.bagSelect:SetSelectedValue(nkWSetup.sets[selSet].bag, true)
  uiElements.charWindow.bankSelect:SetSelectedValue(nkWSetup.sets[selSet].bank, true)
  uiElements.charWindow.roleSelect:SetSelectedValue(nkWSetup.sets[selSet].role, true)
  uiElements.charWindow.iconSelect:SetSelectedValue(nkWSetup.sets[selSet].icon, true)
  uiElements.charWindow.linkSelect:SetSelectedValue(nkWSetup.sets[selSet].linkedSet, true)
  
  if uiElements.charWindow.nkRBSelect ~= nil and uiElements.charWindow.nkRBSelect:GetVisible() == true then
    if nkWSetup.sets[selSet].nkrebuff == nil then nkWSetup.sets[selSet].nkrebuff = -1 end -- initialize for old versions
    uiElements.charWindow.nkRBSelect:SetSelectedValue(nkWSetup.sets[selSet].nkrebuff, true)
  end
  
  if uiElements.charWindow.kAlertMainSelect ~= nil and uiElements.charWindow.kAlertMainSelect:GetVisible() == true then
    if nkWSetup.sets[selSet].kaMain == nil then nkWSetup.sets[selSet].kaMain = -1 end -- initialize for old versions
    uiElements.charWindow.kAlertMainSelect:SetSelectedValue(nkWSetup.sets[selSet].kaMain, true)
    
    if nkWSetup.sets[selSet].kaSub == nil then nkWSetup.sets[selSet].kaSub = -1 end -- initialize for old versions
    uiElements.charWindow.kAlertSubSelect:SetSelectedValue(nkWSetup.sets[selSet].kaSub, true)
  end
  
  for k, v in pairs(uiElements.charWindow.checkboxes) do
    if nkWSetup.sets[selSet].items[k] == nil then
      v:SetChecked(false)
    else
      v:SetChecked(true)
    end
  end
  
  uiElements.charWindow.yellEdit:SetText(nkWSetup.sets[selSet].yell)
  
end

function _internal.updateRebuffSelectValues(target) -- update nkRebuff selection combo

  local addonList = Inspect.Addon.List()
  
  if addonList.nkRebuff ~= nil then 
    if nkWSetup.sets[privateVars.uiSelectedSet].nkrebuff == nil then nkWSetup.sets[privateVars.uiSelectedSet].nkrebuff = -1 end -- initialise for existing sets
    
    local selValues = { { label = privateVars.langTexts.deactivated, value = -1 } }
  
    for idx = 1, #nkRBSets, 1 do
      table.insert (selValues, { value = idx, label = nkRBSets[idx].name })
    end
  
    if target.nkRBSelect ~= nil then 
      target.nkRBSelect:SetSelection(selValues)
      target.nkRBSelect:SetSelectedValue(nkWSetup.sets[privateVars.uiSelectedSet].nkrebuff, false)
    end
  end
  
end

function _internal.tooltipInfo (_, tttype, shown, buff)
  
  if tttype == nil then
    if data.lastTTDisplay ~= nil then
      if Inspect.Time.Real() - data.lastTTDisplay > .2 then
        if uiElements.tooltipInfo ~= nil then uiElements.tooltipInfo:SetVisible(false) end
      end
    end
  elseif tttype ~= 'item' and tttype ~= 'itemtype' then
    if uiElements.tooltipInfo ~= nil then uiElements.tooltipInfo:SetVisible(false) end
  else
    data.lastTTDisplay = Inspect.Time.Real()
    --local ttText = ""
    local lines = {}
      
    for key, details in pairs (nkWSetup.sets) do
      if details.items ~= nil then
        for subkey, setItemID in pairs (details.items) do   
          if setItemID == shown then        
            table.insert (lines, { text = details.name })
          end
        end
      end
    end
    
    if #lines ~= 0 then
      if uiElements.tooltipInfo == nil then _fctBuildTooltipInfo() end 
      uiElements.tooltipInfo:SetVisible(true)
      uiElements.tooltipInfo:SetLines(lines)
      uiElements.tooltipInfo:SetWidth(UI.Native.Tooltip:GetWidth() - 10)      
    else
      if uiElements.tooltipInfo ~= nil then uiElements.tooltipInfo:SetVisible(false) end
    end
  end   
  
end